# -*- coding: ascii -*-

###########################################################################
# clive, video extraction utility
# Copyright (C) 2007-2008 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################

## Tkinter based classes for GUI configuration.

import sys
import os
import subprocess
import Tkinter
import tkSimpleDialog
import tkFileDialog
import tkMessageBox

from clive.opts import Options

__all__ = ['Configure']

# TODO: Could use some cleaning up.


## The main GUI class for configuration.
class Configure:

	## Constructor
	def __init__(self):
		self._widgets = {}

	## Creates the widgets and enters the main loop.
	# 
	# Displays the Tkinter based GUI.
	def main(self):
		self._optparser = Options()
		(self._opts, self._args) = self._optparser.parse()
		self._root = Tkinter.Tk()
		self._root.title('Configure clive')
		self._root.geometry('+100+100')
		self._create_widgets()
		self._root.protocol('WM_DELETE_WINDOW', self._on_close)
		self._root.mainloop()

	def _create_widgets(self):
		self._root.rowconfigure(0, weight=1)
		self._root.columnconfigure(0, weight=1)

		# General settings
		g = _new_group(self._root, name='General Settings',
			row=0, col=0, sticky='nwse')

		a = [
			('verbose', 'Be verbose', None),
			('simulate', 'Simulate only', None),
			('log', 'Enable logging', None),
			('paste', 'Paste input from system clipboard only',
				self._on_paste),
		]

		for (i, r) in enumerate(a):
			(name, text, cmd) = r
			self._widgets[name] = Tkinter.IntVar()
			self._widgets[name].set(self._opts.__dict__[name]) # Set init.val.
			w = Tkinter.Checkbutton(g, text=text,
				variable=self._widgets[name], command=cmd)
			#pady=[0,5][i%2]
			w.grid(row=i, column=0, padx=5, pady=0, sticky='w')

		i = i + 1

		# Buttons
		a = [
			('Connection', self._on_connection),
			('Output', self._on_output),
			('Playback', self._on_playback),
			('Re-encode', self._on_rencode),
			('Clipboard', self._on_clipboard),
		]

		g = _new_group(self._root, name='More', row=i, col=0, sticky='nwse')

		for (c, b) in enumerate(a):
			(text, cmd) = b
			w = Tkinter.Button(g, text=text, command=cmd, width=10)
			w.grid(row=i, column=c, padx=5, pady=5, sticky='nw')

		i = i + 1

		Tkinter.Button(self._root, text='Close',
			command=self._on_close, width=10) \
			.grid(row=i, column=0, padx=5, pady=5, sticky='ne')

	def _on_paste(self):
		if self._widgets['paste'].get():
			if sys.platform != 'win32':
				if self._opts.xclip == '': # Check if xclip is set
					self._on_clipboard() # No, prompt to set it
					if self._opts.xclip == '': # Check again
						self._widgets['paste'].set(0) # Disable
			else:
				try:
					import win32clipboard
					import win32con
				except ImportError:
					self._on_clipboard()
					self._widgets['paste'].set(0)

	def _on_connection(self):
		ConnectionDialog(self._root, self._opts)
	
	def _on_output(self):
		OutputDialog(self._root, self._opts)
	
	def _on_playback(self):
		PlaybackDialog(self._root, self._opts)
	
	def _on_rencode(self):
		RencodeDialog(self._root, self._opts)

	def _on_clipboard(self):
		ClipboardDialog(self._root, self._opts)
	
	def _on_close(self):
		for w in self._widgets: # Save user input (checkbuttons)
			self._opts.__dict__[w] = self._widgets[w].get()
		self._optparser.write_config(self._opts)
		self._root.destroy()


## The class that displays the connection options
class ConnectionDialog(tkSimpleDialog.Dialog):

	## Constructor
	#
	# Constructs and displays the dialog. See tkSimpleDialog.Dialog
	# for Tkinter documentation.
	#
	# \param self The object pointer
	# \param parent The parent object
	# \param opts The program options object
	def __init__(self, parent, opts):
		self._opts = opts
		tkSimpleDialog.Dialog.__init__(self, parent, 'Connection Settings')

	## Creates the dialog widgets and sets their initial values.
	def body(self, root):
		self._root = root
		self._widgets = {}

		# Proxy
		g = _new_group(root, name='Proxy', row=0, col=0, sticky='nw')

		a = [
			'Direct connection / Auto-detect proxy (http_proxy)',
			'Manual proxy configuration:', 
		]

		self._widgets['proxy'] = Tkinter.StringVar()
		self._widgets['proxy_rvar'] = Tkinter.IntVar()
		self._widgets['proxy'].set(self._opts.proxy)

		if self._widgets['proxy'].get():
			self._widgets['proxy_rvar'].set(1)

		for (i, text) in enumerate(a):
			w = Tkinter.Radiobutton(g, text=text,
				variable=self._widgets['proxy_rvar'],
					value=i, command=self._on_proxy)
			w.grid(row=i, column=0, padx=5, pady=0, sticky='nw')

		i = i + 1

		w = Tkinter.Entry(g, textvariable=self._widgets['proxy'], width=40)
		w.grid(row=i, column=0, padx=10, pady=5, sticky='nwe')
		self._widgets['proxy_entry'] = w

		i = i + 1

		Tkinter.Label(g,
			text='NOTE: http_proxy env. variable is used ' \
				'automatically if detected') \
				.grid(row=i, column=0, padx=10, pady=5, sticky='nw')

		# User-agent
		g = _new_group(root, name='User-agent',
			row=0, col=1, sticky='nwe')

		a = [
			'Default (random)',
			'User-defined:',
		]

		self._widgets['user_agent'] = Tkinter.StringVar()
		self._widgets['user_agent_rvar'] = Tkinter.IntVar()
		self._widgets['user_agent'].set(self._opts.user_agent)

		if self._widgets['user_agent'].get():
			self._widgets['user_agent_rvar'].set(1)

		for (i, text) in enumerate(a):
			w = Tkinter.Radiobutton(g, text=text,
				variable=self._widgets['user_agent_rvar'],
					value=i, command=self._on_user_agent)
			w.grid(row=i, column=0, padx=5, pady=0, sticky='nw')

		i = i + 1

		w = Tkinter.Entry(g,
			textvariable=self._widgets['user_agent'], width=25)
		w.grid(row=i, column=0, padx=10, pady=5, sticky='nwe')
		self._widgets['user_agent_entry'] = w

		# Throttle limit
		g = _new_group(root, name='Throttle limit', row=1, col=0, sticky='nwe')

		self._widgets['throttle'] = Tkinter.StringVar()
		self._widgets['throttle_rvar'] = Tkinter.IntVar()

		try:
			v = int(self._opts.throttle) / 1024
			if v == 0:
				v = ''
		except:
			v = ''
		self._widgets['throttle'].set(v)

		if self._widgets['throttle'].get():
			self._widgets['throttle_rvar'].set(1)

		a = [
			'Disabled (unlimited)',
			'Limit transfer rate to KB/s:',
		]

		for (i, text) in enumerate(a):
			w = Tkinter.Radiobutton(g, text=text,
				variable=self._widgets['throttle_rvar'],
					value=i, command=self._on_throttle)
			w.grid(row=i, column=0, padx=5, pady=0, sticky='nw')

		i = i + 1

		w = Tkinter.Entry(g, textvariable=self._widgets['throttle'], width=20)
		w.grid(row=i, column=0, padx=10, pady=5, sticky='nwe')
		self._widgets['throttle_entry'] = w

		# Other
		g = _new_group(root, name='Other', row=1, col=1, sticky='nwe')

		self._widgets['gzip'] = Tkinter.IntVar()

		w = Tkinter.Checkbutton(g, text='Use compressed HTTP streams',
			variable=self._widgets['gzip'])
		w.grid(row=0, column=0, padx=5, pady=5, sticky='nw')

		self._set_initial_values()

	## Applies changes
	def apply(self):
		self._opts.proxy = self._widgets['proxy'].get()

		self._opts.user_agent = self._widgets['user_agent'].get()

		v = self._widgets['throttle'].get()
		if v != '':
			try: v = int(v)*1024
			except ValueError: v = 0
		self._opts.throttle = v

		self._opts.gzip = self._widgets['gzip'].get()

	def _set_initial_values(self):
		self._on_proxy()
		self._on_user_agent()
		self._on_throttle()
		self._widgets['gzip'].set(self._opts.gzip)
	
	def _on_proxy(self):
		if not self._widgets['proxy_rvar'].get():
			self._widgets['proxy'].set('') # Reset and disable
			state = 'disabled'
		else:
			state = 'normal'

		self._widgets['proxy_entry'].config(state=state)
	
	def _on_user_agent(self):
		if not self._widgets['user_agent_rvar'].get():
			self._widgets['user_agent'].set('') # Reset and disable
			state = 'disabled'
		else:
			state = 'normal'

		self._widgets['user_agent_entry'].config(state=state)
	
	def _on_throttle(self):
		if not self._widgets['throttle_rvar'].get():
			self._widgets['throttle'].set('') # Reset and disable
			state = 'disabled'
		else:
			state = 'normal'

		self._widgets['throttle_entry'].config(state=state)
		

## The class that displays the output options
class OutputDialog(tkSimpleDialog.Dialog):

	## Constructor
	#
	# Constructs and displays the dialog. See tkSimpleDialog.Dialog
	# for Tkinter documentation.
	#
	# \param self The object pointer
	# \param parent The parent object
	# \param opts The program options object
	def __init__(self, parent, opts):
		self._opts = opts
		tkSimpleDialog.Dialog.__init__(self, parent, 'Output Settings')

	## Creates the dialog widgets and sets their initial values.
	def body(self, root):
		self._root = root
		self._widgets = {}

		# Output file exists action
		g = _new_group(root, name='If output file exists',
			row=0, col=0, sticky='nwe')

		a = [
			('', 'Skip video'),
			('overwrite', 'Overwrite existing video file'),
			('rename', 'Rename (give video new) output filename'),
		]

		self._widgets['exists'] = Tkinter.StringVar()

		for (i, r) in enumerate(a):
			(name, text) = r
			w = Tkinter.Radiobutton(g, text=text,
				variable=self._widgets['exists'], value=name)
			w.grid(row=i, column=0, padx=5, pady=0, sticky='nw')

		v = ''
		if self._opts.overwrite:
			v = 'overwrite'
		elif self._opts.rename:
			v = 'rename'
		self._widgets['exists'].set(v)

		# Prefix
		g = _new_group(self._root, name='Write extracted videos to',
			row=0, col=1, sticky='nwe')

		a = [
			'Current work directory',
			'Directory:',
		]

		self._widgets['prefix'] = Tkinter.StringVar()
		self._widgets['prefix_rvar'] = Tkinter.IntVar()
		self._widgets['prefix'].set(self._opts.prefix)

		if self._widgets['prefix'].get():
			self._widgets['prefix_rvar'].set(1)

		for (i, text) in enumerate(a):
			w = Tkinter.Radiobutton(g, text=text,
				variable=self._widgets['prefix_rvar'],
					value=i, command=self._on_prefix)
			w.grid(row=i, column=0, padx=5, pady=0, sticky='nw')

		i = i + 1

		w = Tkinter.Entry(g, textvariable=self._widgets['prefix'], width=40)
		w.grid(row=i, column=0, padx=10, pady=5, sticky='nwe')
		self._widgets['prefix_entry'] = w

		w = Tkinter.Button(g, text='...', command=self._on_browse_prefix)
		w.grid(row=i, column=1, padx=5, pady=5, sticky='ne')
		self._widgets['prefix_browse'] = w

		# Output mask
		g = _new_group(self._root, name='Output mask',
			row=1, col=0, sticky='nwe')

		a = [
			'Default mask (A-Za-z0-9)',
			'Disable explicitly',
			'User-defined python method from ~/.clive/userdef.py',
			'User-defined mask:',
		]

		self._widgets['output_mask'] = Tkinter.StringVar()
		self._widgets['output_mask_rvar'] = Tkinter.IntVar()
		self._widgets['output_mask'].set(self._opts.output_mask)

		v = 3
		for (i,t) in enumerate([ '', 'off', '<userdef>' ]):
			if self._opts.output_mask == t:
				v = i

		self._widgets['output_mask_rvar'].set(v)

		for (i, text) in enumerate(a):
			w = Tkinter.Radiobutton(g, text=text,
				variable=self._widgets['output_mask_rvar'],
					value=i, command=self._on_output_mask)
			w.grid(row=i, column=0, padx=5, pady=0, sticky='nw')

		i = i + 1

		w = Tkinter.Entry(g,
			textvariable=self._widgets['output_mask'], width=20)
		w.grid(row=i, column=0, padx=10, pady=5, sticky='nwe')
		self._widgets['output_mask_entry'] = w

		# Output format
		g = _new_group(self._root, name='Output format',
			row=1, col=1, sticky='nwe')

		a = [
			'Default format (%t.%e)',
			'User-defined format:',
		]

		self._widgets['output_fmt'] = Tkinter.StringVar()
		self._widgets['output_fmt_rvar'] = Tkinter.IntVar()
		self._widgets['output_fmt'].set(self._opts.output_fmt)
		self._widgets['output_fmt_rvar'].set(self._opts.output_fmt != '')

		for (i, text) in enumerate(a):
			w = Tkinter.Radiobutton(g, text=text,
				variable=self._widgets['output_fmt_rvar'],
					value=i, command=self._on_output_fmt)
			w.grid(row=i, column=0, padx=5, pady=0, sticky='nw')

		i = i + 1

		w = Tkinter.Entry(g,
			textvariable=self._widgets['output_fmt'], width=20)
		w.grid(row=i, column=0, padx=10, pady=5, sticky='nwe')
		self._widgets['output_fmt_entry'] = w

		i = i + 1

		Tkinter.Label(g, text='%t=title, %h=host, %i=id, %e=extension') \
			.grid(row=i, column=0, padx=10, pady=5, sticky='nw')

		self._set_initial_values()

	## Applies changes
	def apply(self):
		# Output file exists action
		v = self._widgets['exists'].get()
		if v == 'overwrite':
			self._opts.overwrite = 1
			self._opts.rename = 0
		elif v == 'rename':
			self._opts.overwrite = 0
			self._opts.rename = 1
		else:
			self._opts.overwrite = 0
			self._opts.rename = 0

		# Prefix
		self._opts.prefix = self._widgets['prefix'].get()

		# Output mask
		d = ['', 'off', '<userdef>', self._widgets['output_mask'].get()]
		self._opts.output_mask = d[self._widgets['output_mask_rvar'].get()]

		# Output format
		v = self._widgets['output_fmt_rvar'].get()
		v = ['',self._widgets['output_fmt'].get()][v]
		self._opts.output_fmt = v

	def _set_initial_values(self):
		v = ''
		if self._opts.overwrite: v='overwrite'
		elif self._opts.rename: v='rename'
		self._widgets['exists'].set(v)

		self._on_prefix()
		self._on_output_mask()
		self._on_output_fmt()
	
	def _on_prefix(self):
		if not self._widgets['prefix_rvar'].get():
			self._widgets['prefix'].set('') # Reset and disable
			state = 'disabled'
		else:
			state = 'normal'

		for n in ['entry', 'browse']:
			self._widgets['prefix_' + n].config(state=state)
	
	def _on_browse_prefix(self):
		path = tkFileDialog.askdirectory(parent=self._root)
		if path: self._widgets['prefix'].set(path)
	
	def _on_output_mask(self):
		if self._widgets['output_mask_rvar'].get() != 3:
			self._widgets['output_mask'].set('') # Reset and disable
			state = 'disabled'
		else:
			state = 'normal'
		self._widgets['output_mask_entry'].config(state=state)
	
	def _on_output_fmt(self):
		if not self._widgets['output_fmt_rvar'].get():
			self._widgets['output_fmt'].set('') # Reset and disable
			state = 'disabled'
		else:
			state = 'normal'
		self._widgets['output_fmt_entry'].config(state=state)


## The class that displays the playback options
class PlaybackDialog(tkSimpleDialog.Dialog):

	## Constructor
	#
	# Constructs and displays the dialog. See tkSimpleDialog.Dialog
	# for Tkinter documentation.
	#
	# \param self The object pointer
	# \param parent The parent object
	# \param opts The program options object
	def __init__(self, parent, opts):
		self._opts = opts
		tkSimpleDialog.Dialog.__init__(self, parent, 'Playback Settings')
	
	## Creates the dialog widgets and sets initial values
	def body(self, root):
		self._root = root
		self._widgets = {}

		g = _new_group(root, name='Play',
			row=0, col=0, sticky='nwse')

		# Create an invisible frame inside the labeled frame so
		# the radiobutton widgets stay neatly packed and padded
		# inside it.
		g = Tkinter.Frame(g) 
		g.grid(row=0, column=0, padx=5, pady=5, sticky='nw')

		a = [
			('off', 'Disabled'),
			('src', 'Extracted video'),
		]

		self._widgets['play'] = Tkinter.StringVar()

		for (i, r) in enumerate(a):
			(val, text) = r
			w = Tkinter.Radiobutton(g, text=text,
				variable=self._widgets['play'], value=val,
				command=self._on_play)
			w.grid(row=i, column=0, padx=5, pady=0, sticky='nw')

		i = i + 1

		g = Tkinter.Frame(g) # Create a frame for "re-encoded format"
		g.grid(row=i, column=0, sticky='nw', padx=5, pady=0)

		w = Tkinter.Radiobutton(g, text='Re-encoded video:',
			variable=self._widgets['play'],
				value='other', command=self._on_play)
		w.grid(row=0, column=0, padx=0, pady=0, sticky='nw')

		self._widgets['play_rencoded'] = Tkinter.StringVar()

		w = Tkinter.Entry(g,
			textvariable=self._widgets['play_rencoded'], width=10)
		w.grid(row=0, column=1, padx=5, pady=0, sticky='nw')
		self._widgets['play_rencoded_entry'] = w

		Tkinter.Label(g, text='format, e.g. "mpg" or "avi"..') \
			.grid(row=0, column=2, sticky='nw')

		if self._opts.play != 'off' and self._opts.play != 'src':
			v = 'other'
			self._widgets['play_rencoded'].set(self._opts.play)
		else:
			v = self._opts.play
		self._widgets['play'].set(v)

		# New label frame for "path to player"
		g = _new_group(self._root, name='Path to player',
			row=1, col=0, sticky='nwe')

		self._widgets['player'] = Tkinter.StringVar()
		self._widgets['player'].set(self._opts.player)

		w = Tkinter.Entry(g, textvariable=self._widgets['player'], width=40)
		w.grid(row=0, column=0, padx=10, pady=5, sticky='nwe')
		self._widgets['player_entry'] = w

		w = Tkinter.Button(g, text='...', command=self._on_browse_player)
		w.grid(row=0, column=1, padx=5, pady=5, sticky='nw')
		self._widgets['player_browse'] = w

		if sys.platform != 'win32':
			w = Tkinter.Button(g, text='Autodetect',
				command=self._on_autodetect_player, width=10)
			w.grid(row=0, column=2, padx=5, pady=5, sticky='nw')
			self._widgets['player_autodetect'] = w

		Tkinter.Label(g,
			text='NOTE: Be sure to append at least %i (input file) ' +
			'to the path').grid(row=1, column=0, padx=5, pady=5, sticky='nw',
				columnspan=4)

		self._set_initial_values()

	## Validates input
	def validate(self):
		play = self._widgets['play'].get()
		if play != 'off':
			if play == 'other':
				v = self._widgets['play_rencoded'].get()
				if len(v) == 0:
					tkMessageBox.showwarning('Warning',
						'Set re-encoded video format, e.g. "mpg" or "avi".')
					return 0
			if self._widgets['player'].get() == '':
				tkMessageBox.showwarning('Warning',
					'You have not set path to a player command.')
				return 0
		return 1

	## Applies changes
	def apply(self):
		self._opts.play = self._widgets['play'].get()
		self._opts.player = self._widgets['player'].get()

		if self._opts.play == 'other':
			self._opts.play = self._widgets['play_rencoded'].get()
			self._opts.re_encode = self._opts.play
			if not self._opts.ffmpeg:
				tkMessageBox.showwarning('Warning',
					'You have not set path to ffmpeg. The path needs ' +
					'to be set before re-encoding is possible.\n\n' + 
					'Click "Re-encode" to do that.')

	def _set_initial_values(self):
		self._on_play()
	
	def _on_play(self):
		state = ['disabled','normal'][self._widgets['play'].get() != 'off']

		for n in ['entry', 'browse', 'autodetect']:
			if sys.platform == 'win32' and n == 'autodetect':
				continue
			self._widgets['player_' + n].config(state=state)

		state = ['disabled','normal'][self._widgets['play'].get() == 'other']
		self._widgets['play_rencoded_entry'].config(state=state)

		if self._widgets['play'].get() != 'other':
			self._widgets['play_rencoded'].set('')

	def _on_browse_player(self):
		path = tkFileDialog.askopenfilename(parent=self._root)
		if path: self._widgets['player'].set(path)
	
	def _on_autodetect_player(self):
		a = ['vlc', 'mplayer', 'gmplayer', 'totem', 'xine']
		for p in a:
			p = _locate_bin(p, '%i', quiet=1)
			if len(p) > 0:
				self._widgets['player'].set(p)
				return
		tkMessageBox.showwarning('Warning',
			'Unable to locate a player.\n\nSearched the list of ' \
			'known commands: %s.' % repr(a))


## The class that displays the re-encoding options
class RencodeDialog(tkSimpleDialog.Dialog):

	## Constructor
	#
	# Constructs and displays the dialog. See tkSimpleDialog.Dialog
	# for Tkinter documentation.
	#
	# \param self The object pointer
	# \param parent The parent object
	# \param opts The program options object
	def __init__(self, parent, opts):
		self._opts = opts
		tkSimpleDialog.Dialog.__init__(self, parent, 'Re-encoding Settings')
	
	## Creates the dialog widgets and sets initial values
	def body(self, root):
		self._root = root
		self._widgets = {}

		g = _new_group(root,
			name='Re-encode',
			row=0, col=0, sticky='nwse')

		# Create another frame, see PlaybackDialog for details
		g = Tkinter.Frame(g)
		g.grid(row=0, column=0, padx=5, pady=5, sticky='nw')

		a = [
			('off', 'Disabled'),
		]

		self._widgets['re_encode'] = Tkinter.StringVar()

		for (i, r) in enumerate(a):
			(val, text) = r
			w = Tkinter.Radiobutton(g, text=text,
				variable=self._widgets['re_encode'], value=val,
				command=self._on_rencode)
			w.grid(row=i, column=0, padx=5, pady=0, sticky='nw')

		i = i + 1

		g = Tkinter.Frame(g) # Create a frame for "format'
		g.grid(row=i, column=0, sticky='nw', padx=5, pady=0)

		w = Tkinter.Radiobutton(g, text='Format:',
			variable=self._widgets['re_encode'],
				value='format', command=self._on_rencode)
		w.grid(row=0, column=0, padx=0, pady=0, sticky='nw')

		self._widgets['rencode_format'] = Tkinter.StringVar()

		w = Tkinter.Entry(g, 
			textvariable=self._widgets['rencode_format'], width=10)
		w.grid(row=0, column=1, padx=5, pady=0, sticky='nw')
		self._widgets['rencode_format_entry'] = w

		Tkinter.Label(g, text='e.g. "mpg" or "avi"..') \
			.grid(row=0, column=2, sticky='nw')

		if self._opts.play != 'off' and self._opts.play != 'src':
			self._opts.re_encode = self._opts.play

		if self._opts.re_encode != 'off':
			v = 'format'
			self._widgets['rencode_format'].set(self._opts.re_encode)
		else:
			v = self._opts.re_encode
		self._widgets['re_encode'].set(v)

		# Path to ffmpeg group
		g = _new_group(self._root, name='Path to ffmpeg',
			row=1, col=0, sticky='nwe')

		self._widgets['ffmpeg'] = Tkinter.StringVar()
		self._widgets['ffmpeg'].set(self._opts.ffmpeg)

		w = Tkinter.Entry(g, textvariable=self._widgets['ffmpeg'], width=40)
		w.grid(row=0, column=0, padx=10, pady=5, sticky='nwe')
		self._widgets['ffmpeg_entry'] = w

		w = Tkinter.Button(g, text='...', command=self._on_browse_ffmpeg)
		w.grid(row=0, column=1, padx=5, pady=5, sticky='nw')
		self._widgets['ffmpeg_browse'] = w

		if sys.platform != 'win32':
			w = Tkinter.Button(g, text='Autodetect',
				command=self._on_autodetect_ffmpeg, width=10)
			w.grid(row=0, column=2, padx=5, pady=5, sticky='nw')
			self._widgets['ffmpeg_autodetect'] = w

		Tkinter.Label(g, text='NOTE: Be sure to append ' +
			'at least "-y -i %i %o" to the path').grid(row=1, column=0,
				padx=5, pady=5, sticky='nw', columnspan=4)

		self._set_initial_values()

	## Validates input
	def validate(self):
		re_encode = self._widgets['re_encode'].get()
		if re_encode != 'off':
			v = self._widgets['rencode_format'].get()
			if len(v) == 0:
				tkMessageBox.showwarning('Warning',
					'Set format for re-encoded videos, e.g. "mpg" or ".avi"')
				return 0
			if self._widgets['ffmpeg'].get() == '':
				tkMessageBox.showwarning('Warning',
					'You have not set path to ffmpeg command.')
				return 0
		return 1

	## Applies changes
	def apply(self):
		self._opts.re_encode = self._widgets['re_encode'].get()
		self._opts.ffmpeg = self._widgets['ffmpeg'].get()

		if self._opts.re_encode != 'off':
			self._opts.re_encode = self._widgets['rencode_format'].get()

	def _set_initial_values(self):
		self._on_rencode()
	
	def _on_rencode(self):
		state = ['disabled','normal'] \
			[self._widgets['re_encode'].get() != 'off']

		for n in ['entry', 'browse', 'autodetect']:
			if sys.platform == 'win32' and n == 'autodetect':
				continue
			self._widgets['ffmpeg_' + n].config(state=state)

		self._widgets['rencode_format_entry'].config(state=state)

		if self._widgets['re_encode'].get() != 'format':
			self._widgets['rencode_format'].set('')

	def _on_browse_ffmpeg(self):
		path = tkFileDialog.askopenfilename(parent=self._root)
		if path: self._widgets['ffmpeg'].set(path)
	
	def _on_autodetect_ffmpeg(self):
		self._widgets['ffmpeg'].set(_locate_bin('ffmpeg', '-y -i %i %o'))


## The class that displays the system clipboard options
class ClipboardDialog(tkSimpleDialog.Dialog):

	## Constructor
	#
	# Constructs and displays the dialog. See tkSimpleDialog.Dialog
	# for Tkinter documentation.
	#
	# \param self The object pointer
	# \param parent The parent object
	# \param opts The program options object
	def __init__(self, parent, opts):
		self._opts = opts
		tkSimpleDialog.Dialog.__init__(self, parent, 'Clipboard Settings')
	
	## Creates the dialog widgets and sets initial values
	def body(self, root):
		self._root = root
		self._widgets = {}

		g = _new_group(root, name='', row=0, col=0, sticky='nwse')

		if sys.platform != 'win32':
			self._non_mswin(g)
		else:
			self._mswin(g)

	## Validates input
	def validate(self):
		if sys.platform != 'win32':
			if len(self._widgets['xclip'].get()) == 0:
				tkMessageBox.showwarning('Warning',
					'You have not set path to xclip')
				return 0
		return 1

	## Applies changes
	def apply(self):
		if sys.platform != 'win32':
			self._opts.xclip = self._widgets['xclip'].get()

	def _non_mswin(self, g):
		Tkinter.Label(g, text='Path to xclip:').grid(row=0, column=1,
			padx=5, pady=5, sticky='w')

		self._widgets['xclip'] = Tkinter.StringVar()
		self._widgets['xclip'].set(self._opts.xclip)

		self._widgets['xclip_entry'] = Tkinter.Entry(g,
			textvariable=self._widgets['xclip'])

		self._widgets['xclip_entry'].grid(row=0, column=2,
			padx=5, pady=5, sticky='we')

		w = Tkinter.Button(g, text='Autodetect',
			command=self._on_autodetect_xclip, width=10)

		w.grid(row=0, column=3, padx=5, pady=5, sticky='e')

		Tkinter.Label(g,
			text='NOTE: Be sure to append the "-o" option to the path') \
				.grid(row=1, column=0, padx=5, pady=5,
					sticky='nw', columnspan=4)

	def _mswin(self, g):
		try:
			import win32clipboard
			import win32con
			text = 'Required modules found. Clipboard support enabled.'
		except ImportError:
			text = 'Required modules not found. ' \
				'Clipboard support disabled.\n' \
				'Visit http://sf.net/projects/pywin32 for the modules.'

		Tkinter.Label(g, text=text).grid(row=0, column=0,
			padx=5, pady=5, sticky='nwse')

	def _on_autodetect_xclip(self):
		self._widgets['xclip'].set(_locate_bin('xclip', '-o'))


## Creates a new label frame group
def _new_group(parent, name, row, col, sticky, padx=5, pady=5, cspan=None):
	g = Tkinter.LabelFrame(parent, text=name)
	g.grid(row=row, column=col, padx=padx, pady=pady, sticky=sticky,
		columnspan=cspan)

	g.grid_rowconfigure(0, weight=1)
	g.grid_columnconfigure(0, weight=1)

	return g

## Locates an executable using the `which' command
def _locate_bin(cmd, cmd_opts, quiet=0):
	path = ''
	pipe = subprocess.Popen('which %s 2>/dev/null' % cmd,
		shell=1, stdout=subprocess.PIPE).stdout
	data = pipe.read().rstrip('\n')
	if not len(data):
		if not quiet:
			tkMessageBox.showwarning('Warning', 'Unable to locate ' \
				'the `%s` command.' % cmd)
	else:
		path = data + ' ' + cmd_opts
	
	return path


if __name__ == '__main__':
	Configure().main()


