# -*- coding: ascii -*-

###########################################################################
# clive, video extraction utility
# Copyright (C) 2007-2008 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################

## The classes for parsing runtime configuration and command line options

import os
import os.path
import sys
import random
import time
import gzip
import errno
import types

from optparse import OptionParser, OptionGroup

import urlgrabber as _urlgrabber
import clive as _clive

try:
	import Tkinter
	_tkinter_avail = 1
except:
	_tkinter_avail = 0

try:
	import win32clipboard
	import win32con
	_mswin_clipboard_avail = 1
except ImportError:
	_mswin_clipboard_avail = 0

__all__ = ['Options']


## The class for parsing runtime configuration and command line options
class Options:
	_rcfile = 'config.py'
	_lastfile = 'last.log'
	_historyfile = 'history.log'
	_rcdir = '.clive'

	## Constructor
	def __init__(self):
		self._version = '%s (%s; python/%s; urlgrabber/%s)' % (
			_clive.__version_str__,
			sys.platform,
			sys.version.split()[0],
			_urlgrabber.__version__
		)

		if sys.platform == 'win32':
			self._rcdir = self._rcdir.strip('.')

		self._full_rcdir = os.path.join(os.path.expanduser('~'), self._rcdir)

		try: # Make sure ~/.clive exists
			os.makedirs(self._full_rcdir)
		except OSError, e:
			if e.errno == errno.EEXIST: pass
			else: raise

		## Path to runtime configuration file
		self._rcfile = os.path.join(self._full_rcdir, self._rcfile)
		self._lastfile = os.path.join(self._full_rcdir, self._lastfile)
		self._historyfile = os.path.join(self._full_rcdir, self._historyfile)

		self._warnings = []

	## Parses runtime configuration and command line options
	def parse(self):
		self._parseopts()
		return (self.opts, self.args)

	## Prints out some of the program options to stdout
	#
	# \param self The object pointer
	# \param say The stdout print method
	def show(self, say):
		a = ['configure', 'conf_write', 'history', 'log_clear']
		for n in a:
			if self.opts.__dict__[n]:
				for w in self._warnings:
					say(w)
				return

		if self.opts.rename: if_exists = 'rename'
		elif self.opts.overwrite: if_exists = 'overwrite'
		else: if_exists = 'skip'

		onoff = ['off','on']

		say('%s\n[log:%s] [play:%s] [encode:%s] [throttle:%s] [exists:%s]' % (
			self._version,
			(onoff[self.opts.log]),
			self.opts.play,
			self.opts.re_encode,
			(['off','%.0fK/s' % (self.opts.throttle/1024)] \
				[self.opts.throttle > 0]),
			if_exists)
		)

		for w in self._warnings:
			say(w)

	## (Re)writes configuration file with defaults
	#
	# \param self The object pointer
	# \param say The stdout print method
	def rewrite_config(self, say):
		if os.path.exists(self._rcfile):
			a = raw_input('rewrite existing config file? (y/N): ')
			if a.lower() != 'y':
				return
			try:
				os.remove(self._rcfile)
			except OSError:
				if e.errno == errno.EEXIST: pass
				else: raise
		(opts, args) = Options().parse()
		say('writing %s...' % self._rcfile)
		self.write_config(opts, keep_empty_strings=1)

	## Writes runtime configuration file
	#
	# \param self The object pointer
	# \param opts The options object
	# \param keep_empty_strings Boolean for keeping empty strings or not
	def write_config(self, opts, keep_empty_strings=0):
		a = [
			'configure', 'conf_write', 'history', 'log_recall', 'emit',
			'check_updates', 'log_clear', 'output_file'
		]

		for n in a: # Remove above from the dict
			del opts.__dict__[n]

		for (n,v) in opts.__dict__.items(): # Remove specific from the dict
			rm = 0
			if n.startswith('_'): # Those that begin with '_'
				rm = 1
			if type(v) == types.StringType and not keep_empty_strings:
				if len(v) == 0: # Empty strings
					rm = 1
			if rm: del opts.__dict__[n]

		for (n,v) in opts.__dict__.items():
			if v == True: v = 1 # Make small tweaks on the output
			if v == False: v = 0
			opts.__dict__[n] = v
			if n == 'throttle':
				if type(opts.throttle) == types.StringType:
					if len(opts.throttle) == 0:
						opts.throttle = 0
				opts.throttle = opts.throttle / 1024

		f = open(self._rcfile, 'w')
		f.write('# NOTE: Generated using --configure\n')
		f.write('{\n')
		for (name, value) in opts.__dict__.items():
			f.write('\t%s:%s,\n' % (repr(name), repr(value)))
		f.write('}\n\n')
		f.close()

	## Writes ~/.clive/last.log file
	#
	# \param self The object pointer
	# \param raw_urls A list of user input URLs
	def write_last_log(self, raw_urls):
		try:
			f = open(self._lastfile, 'w')
		except IOError, e:
			if e.errno == errno.ENOENT: pass
			else: raise
		for u in raw_urls:
			f.write(u + '\n')
		f.close()

	## Reads the contents of the ~/.clive/last.log file
	#
	# \param self The object pointer
	def read_last_log(self):
		data = ''
		try:
			f = open(self._lastfile)
			data = f.read()
			f.close()
		except IOError, e:
			if e.errno == errno.ENOENT: pass
			else: raise
		return data

	## Writes a new entry to ~/.clive/history.log file
	#
	# The log file is a CSV file and uses the line format:
	# "current-time","url","output-filename","file-length"
	#
	# Implements a basic log rotation logic, the file is
	# archived if the length exceeds 1MB.
	#
	# \param self The object pointer
	# \param video An packed object containing various video details
	def write_history(self, video):
		if os.path.exists(self._historyfile): # Archive existing log if:
			if os.path.getsize(self._historyfile) >= (1024*1024): # >= 1MB
				i = 1
				while 1:
					fn = '%s-%d.gz' % (self._historyfile, i)
					if not os.path.exists(fn):
						break
					i = i + 1
				f = open(self._historyfile, 'r')
				gzip.GzipFile(fn, 'w').write(f.read())
				f.close()
				os.remove(self._historyfile)
				
		(url, vurl, hlen, length, fn, reget, exists) = video

		open(self._historyfile, 'a').write(
			'"%s","%s","%s","%s"\n' % (
				time.strftime('%Y-%m-%dT%H:%M:%S'),
				url, os.path.basename(fn), hlen
			)
		)

	## Clears all logs from ~/.clive
	#
	# \param self The object pointer
	# \param only_last_log 0/1: whether to clear only ~/.clive/last.log
	def log_clear(self, only_last_log=0):
		a = [self._lastfile]
		if not only_last_log:
			a.append(self._historyfile)
		for file in a:
			try:
				os.remove(file)
			except OSError, e:
				if e.errno == errno.ENOENT: pass
				else: raise

	# Non-public

	def _parseopts(self):
		try:
			self._config = eval(open(self._rcfile).read())
		except IOError, e:
			if e.errno == errno.ENOENT: pass
			else: raise
			self._config = None

		p = OptionParser(
			usage = '%prog [options] URLs',
			description = _clive.__doc__,
			version = self._version + '\n' + _clive.__notice__
		)

		goutput = OptionGroup(p, 'output', None)
		gconn = OptionGroup(p, 'connection', None)
		gplayback = OptionGroup(p, 'playback', None)
		gencode = OptionGroup(p, 're-encode', None)
		gclip = OptionGroup(p, 'clipboard', None)
		glog = OptionGroup(p, 'log', None)

		p.add_option_group(goutput)
		p.add_option_group(gconn)
		p.add_option_group(gplayback)
		p.add_option_group(gencode)
		p.add_option_group(gclip)
		p.add_option_group(glog)

		# optgroup, varname, lopt, sopt, action, const
		# choices, desc, callback, type, metavar, default

		lookup = [
			(p, 'simulate', '--simulate', '-s', 'store_true',
				None, None, 'simulate only, do not extract any videos',
				None, None, None, self._getconf('simulate',0,1)),
			(p, 'emit', '--emit', '-e', 'store_true', None, None,
				'print out found extraction urls only',
				None, None, None, 0),
			(p, 'verbose', '--quiet', '-q', 'store_false', None, None,
				'be quiet', None, None, None, self._getconf('verbose',1,1)),
			(p, 'configure', '--configure', None, 'store_true',
				None, None, 'configure using graphical user interface',
				None, None, None, 0),
			(p, 'conf_write', '--conf-write', None, 'store_true',
				None, None, '(re)write config file with defaults',
				None, None, None, 0),
			(p, 'check_updates', '--check-updates', None, 'store_true',
				None, None, 'check for new updates', None, None, None, 0),

			(goutput, 'overwrite', '--overwrite', '-o', 'store_true', None,
				None, 'overwrite already existing output files',
				None, None, None, self._getconf('overwrite',0,1)),
			(goutput, 'rename', '--rename', '-r', 'store_true', None, None,
				'rename (give video new) output filename',
				None, None, None, self._getconf('rename',0,1)),
			(goutput, 'output_file', '--output', '-O', 'store',
				None, None, 'write videos to file', None, None, 'FN', None),
			(goutput, 'prefix', '--prefix', None, 'store', None, None,
				'write videos to PREFIX',
				None, None, 'PREFIX', self._getconf('prefix',os.getcwd())),
			(goutput, 'output_mask', '--output-mask', None, 'store',
				None, None, 'use MASK for re.sub method (default:"A-Za-z0-9")',
				None, None, 'MASK', self._getconf('output_mask','A-Za-z0-9')),
			(goutput, 'output_fmt', '--output-fmt', None, 'store',
				None, None, 'use FMT for output file format (default:"%t.%e")',
				None, None, 'FMT', self._getconf('output_fmt','%t.%e')),
			(goutput, 'output_mask', '--no-mask', None, 'store_const',
				'off', None, 'explicitly turn off output mask', None, None,
				None, self._getconf('output_mask','A-Za-z0-9')),

			(gconn, 'proxy', '--proxy', None, 'store', None, None,
				'set http proxy (http_proxy env.var. is used autom.)',
				None, None, 'PROXY', self._getconf('proxy')),
			(gconn, 'user_agent', '--user-agent', None, 'store', None, None,
				'set http user-agent (default:random)', None, None, 'AGENT',
				self._getconf('user_agent',self._gen_user_agent())),
			(gconn, 'throttle', '--throttle', None, 'store', None, None,
				'limit transfer rate to LIMIT KB/s (default:unlimited)',
				None, 'int', 'LIMIT', self._getconf('throttle',0)),
			(gconn, 'gzip', '--no-gzip', None, 'store_false',
				None, None,	'explicitly turn off http stream compression',
				None, None, None, self._getconf('gzip',1,1)),

			(gplayback, 'player', '--player', None, 'store', None, None,
				'path to player a command (with any opts.)', None, None,
				'PATH', self._getconf('player')),
			(gplayback, 'play', '--play', '-p', 'store',
				None, None,# ['off','src','s','mpg','m','avi','a'],
				'play video format (src, mpg, avi..)', None, None, 'FMT',
				self._getconf('play')),
			(gplayback, 'play', '--no-play', None, 'store_const',
				'off', None, 'explicitly turn off subsequent playback',
				None, None, None, self._getconf('play')),

			(gencode, 'ffmpeg', '--ffmpeg', None, 'store', None, None,
				'path to ffmpeg command (with any opts.)', None, None,
				'PATH', self._getconf('ffmpeg')),
			(gencode, 're_encode', '--re-encode', None, 'store',
				None, None,# ['off','mpg','m', 'avi','a'],
				're-encode video to format (mpg, avi)', None, None, 'FMT',
				self._getconf('re_encode')),
			(gencode, 're_encode', '--no-encode', None, 'store_const',
				'off', None, 'explicitly turn off subsequent re-encoding',
				None, None, None, self._getconf('re_encode')),

			(gclip, 'xclip', '--xclip', None, 'store', None, None,
				'path to xclip command (with "-o" option)', None, None, 'PATH',
				self._getconf('xclip')),
			(gclip, 'paste', '--paste', '-c',
				'store_true', None, None, 'paste input from clipboard',
				None, None, None, self._getconf('paste',0,1)),

			(glog, 'log_recall', '--log-recall', '-R', 'store_true', None, None,
				'recall url batch from last logged runtime', None, None,
				None, 0),
			(glog, 'log_clear', '--log-clear', '-L', 'store_true',
				None, None, 'clear all log information', None, None, None, 0),
			(glog, 'history', '--history', None, 'store_true', None, None,
				'browse log using graphical user interface', None, None,
				None, 0),
			(glog, 'log', '--no-log', None, 'store_false', None, None,
				'explicitly turn off all logging', None, None, None,
				self._getconf('log',1,1)),
		]

		for (optgroup, varname, lopt, sopt, action, \
				const, choices, desc, callback, \
				_type, metavar, default) in lookup:
			optgroup.add_option(
				lopt, sopt,
				dest=varname,
				action=action,
				const=const,
				choices=choices,
				callback=callback,
				type=_type,
				help=desc,
				metavar=metavar,
				default=default
			)

		(self.opts, self.args) = p.parse_args()

		# Exceptions / tweaks

		proxy = os.environ.get('http_proxy')
		if proxy and not self.opts.proxy:
			self.opts.proxy = proxy

		if self.opts.prefix or len(self.opts.prefix) == 0:
			if not os.path.exists(self.opts.prefix):
				self.opts.prefix = os.getcwd()
				self._warnings.append('warn: prefix path does not exist; ' +
					'using current dir instead')

		if len(self.opts.output_mask) == 0:
			self._warnings.append('warn: --output-mask cannot be ' +
				'an empty string, using default mask')
			self.opts.output_mask = 'A-Za-z0-9'

		if len(self.opts.output_fmt) == 0:
			self._warnings.append('warn: --output-fmt cannot be ' +
				'an empty string, using default format')
			self.opts.output_fmt = '%t.%e'

		if len(self.opts.user_agent) == 0:
			self._warnings.append('warn: --user-agent cannot be ' +
				'an empty string, using a random string')
			self.opts.user_agent = self._gen_user_agent()

		# kb -> bytes for urlgrabber

		self.opts.throttle = self.opts.throttle * 1024

		# --play, --player, --ffmpeg, --re-encode

		if self.opts.play == None:
			self.opts.play = 'off'

		if self.opts.re_encode == None:
			self.opts.re_encode = 'off'

		if self.opts.player == None or len(self.opts.player) == 0:
			if self.opts.play != 'off':
				self._warnings.append('warn: --player not used, ' +
					'playback disabled')
			self.opts.play = 'off'

		if self.opts.ffmpeg == None or len(self.opts.ffmpeg) == 0:
			if self.opts.re_encode != 'off':
				self._warnings.append('warn: --ffmpeg not used, ' +
					're-encoding disabled')
			self.opts.re_encode = 'off'

		if self.opts.play != 'off' and self.opts.play != 'src':
			if self.opts.ffmpeg and len(self.opts.ffmpeg) > 0:
				self.opts.re_encode = self.opts.play
			else:
				self._warnings.append('warn: --ffmpeg not used, ' \
					'playing "src" instead of "%s"' % self.opts.play)
				self.opts.play = 'src'
				self.opts.re_encode = 'off'
				
		if self.opts.player: # Ensure --player was supplied with '%i'
			if self.opts.player.find('%i') == -1:
				if self.opts.play != 'off':
					self._warnings.append('warn: --player does not ' +
						'specify "%i", playback disabled')
					if self.opts.play != 'src':
						self._warnings.append('warn: re-encoding ' +
							'disabled due to the above')
						self.opts.re_encode = 'off'
					self.opts.play = 'off'

		if self.opts.ffmpeg: # Ensure --ffmpeg was supplied with '%i' and '%o'
			for l in ['%i','%o']:
				if self.opts.ffmpeg.find(l) == -1:
					if self.opts.re_encode != 'off':
						self._warnings.append('warn: --ffmpeg does not ' \
							'specify "%s", re-encoding disabled' % l)
						self.opts.re_encode = 'off'
						if self.opts.play != 'off' and self.opts.play != 'src':
							self._warnings.append('warn: due to the above, ' \
								'playing "src" instead of "%s"' % \
									self.opts.play)
							self.opts.play = 'src'
		"""
		print self.opts.play, self.opts.re_encode
		print self.opts.player, self.opts.ffmpeg
		"""
		if self.opts.paste: # Check clipboard options
			if sys.platform != 'win32':
				if not self.opts.xclip:
					self._warnings.append('warn: --xclip not used, ' +
						'disabling --paste')
					self.opts.paste = 0
			else:
				if not _mswin_clipboard_avail:
					self._warnings.append('warn: win32clipboard module ' +
						'not found, disabling --paste')
					self.opts.paste = 0

		# --configure; Replace default values (e.g. user_agent=(random-string))
		# with an empty string. This makes parsing and writing these options
		# less tedious.
		if self.opts.configure or self.opts.conf_write:
			if not self._config:
				self._config = {}
			a = ['user_agent', 'prefix', 'output_mask', 'output_fmt',
				'throttle', 'player', 'ffmpeg', 'xclip', 'proxy']
			for n in a:
				try:
					# Test if the key exists in ~/.clive/config.py
					self.opts.__dict__[n] = self._config[n]
				except KeyError:
					self.opts.__dict__[n] = ''

		# Set various flags
		setattr(self.opts, '_tkinter_avail',
			_tkinter_avail) # --configure et al

		setattr(self.opts, '_mswin_clipboard_avail',
			_mswin_clipboard_avail)

		setattr(self.opts, '_full_rcdir',
			self._full_rcdir)

	def _getconf(self, key, default=None, return_bool=0, choices=None):
		if not self._config:
			return default

		try:
			value = self._config[key]

			if return_bool:
				value = self._istrue(value)
			elif choices:
				try:
					choices.index(value)
				except ValueError:
					raise SystemExit('%s:%s: invalid choice: "%s", ' \
						'choose from %s' % (self._rcfile, key,
							value, repr(choices)))
			return value

		except KeyError:
			return default

	def _istrue(self, value):
		if type(value) == str:
			v = value.lower()
			return (v == 'on' or v == 'yes' or v == '1' or v == 'true')
		else:
			return value == 1
	
	def _gen_user_agent(self, max_pairs=4, max_pair_len=2):
		words = ['clive', 'youtube', 'daily', 'motion',
			'stage', 'video', 'google', 'guba', 'meta', 'cafe']

		a = []

		for w in words:
			i=0; l=max_pair_len
			while 1:
				a.append(w[i:l])
				i=l; l=l+max_pair_len
				if i >= len(w):
					break

		t=''; l=-1

		for i in range(max_pairs):
			while 1:
				j = random.randint(0, len(a)-1)
				if j != l: break
			l=j; t=t+a[j]

		ma = random.randint(0, 2)
		mi = random.randint(0, 9)
		rv = random.randint(0, 9)

		return '%s/%d.%d.%d' % (t, ma, mi, rv)


